"""Test mixins."""

import os
import sys

import six

from django.core.management import call_command
from django.utils.six import StringIO


class CallCommandMixin(object):
    """A mixin to provide command execution shortcuts."""

    def setUp(self):
        """Replace stdin"""
        super(CallCommandMixin, self).setUp()
        self.stdin = sys.stdin

    def tearDown(self):
        sys.stdin = self.stdin

    def import_report(self, path):
        """Import test report from file."""
        with open(path) as fp:
            buf = six.StringIO(fp.read())
        sys.stdin = buf
        out = StringIO()
        call_command("import_aggregated_report", "--pipe", stdout=out)
        return out.getvalue()

    def import_reports(self, folder="reports"):
        """Import reports from folder."""
        path = os.path.join(os.path.dirname(__file__), folder)
        for f in os.listdir(path):
            fpath = os.path.join(path, f)
            if f.startswith(".") or not os.path.isfile(fpath):
                continue
            self.import_report(fpath)

    def import_fail_reports(self, folder="fail-reports"):
        """Import failed reports from folder."""
        path = os.path.join(os.path.dirname(__file__), folder)
        for f in os.listdir(path):
            fpath = os.path.join(path, f)
            if f.startswith(".") or not os.path.isfile(fpath):
                continue
            with self.assertRaisesMessage(SystemExit, '65'):
                self.import_report(fpath)



"""AppConfig for dmarc."""

from django.apps import AppConfig


class DmarcConfig(AppConfig):

    """App configuration."""

    name = "modoboa_dmarc"
    verbose_name = "Modoboa DMARC tools"

    def ready(self):
        from . import handlers



"""DMARC models."""

from django.db import models
from django.utils.translation import ugettext_lazy as _

from modoboa.admin import models as admin_models


POLICY_DISPOSITIONS = [
    ("none", _("None")),
    ("quarantine", _("Quarantine")),
    ("reject", _("Reject"))
]

COMMON_RESULTS = [
    ("none", _("None")),
    ("neutral", _("Neutral")),
    ("pass", _("Pass")),
    ("fail", _("Fail")),
    ("temperror", _("Temporary error")),
    ("permerror", _("Permanent error"))
]

DKIM_RESULTS = COMMON_RESULTS + [
    ("policy", _("Policy"))
]

SPF_RESULTS = COMMON_RESULTS + [
    ("softfail", _("Soft failure"))
]

RECORD_TYPES = [
    ("dkim", "DKIM"),
    ("spf", "SPF"),
]


class Reporter(models.Model):

    """Report issuers."""

    org_name = models.CharField(max_length=100)
    email = models.EmailField(unique=True)

    def __str__(self):
        """Return name and email."""
        return "{} <{}>".format(self.org_name, self.email)


class Report(models.Model):

    """Aggregated reports."""

    report_id = models.CharField(max_length=100)
    reporter = models.ForeignKey(Reporter, on_delete=models.CASCADE)
    start_date = models.DateTimeField()
    end_date = models.DateTimeField()

    # Published policy
    policy_domain = models.CharField(max_length=100)
    policy_adkim = models.CharField(max_length=1)
    policy_aspf = models.CharField(max_length=1)
    policy_p = models.CharField(max_length=10)
    policy_sp = models.CharField(max_length=10)
    policy_pct = models.SmallIntegerField()

    class Meta:
        unique_together = ("reporter", "report_id")

    def __str__(self):
        """Display provider and dates."""
        return "{}: {} -> {}".format(
            self.reporter, self.start_date, self.end_date)


class Record(models.Model):

    """Report records."""

    report = models.ForeignKey(Report, on_delete=models.CASCADE)
    source_ip = models.GenericIPAddressField()
    count = models.IntegerField()

    # Policy evaluated
    disposition = models.CharField(
        max_length=10, choices=POLICY_DISPOSITIONS)
    dkim_result = models.CharField(
        max_length=9, choices=DKIM_RESULTS)
    spf_result = models.CharField(
        max_length=9, choices=SPF_RESULTS)
    header_from = models.ForeignKey(admin_models.Domain,
                                    on_delete=models.CASCADE)
    reason_type = models.CharField(max_length=15, blank=True)
    reason_comment = models.CharField(max_length=100, blank=True)


class Result(models.Model):

    """Record results."""

    record = models.ForeignKey(Record, on_delete=models.CASCADE)
    type = models.CharField(max_length=4, choices=RECORD_TYPES)
    domain = models.CharField(max_length=100)
    result = models.CharField(max_length=9)



"""Import a DMARC aggregated report."""

from __future__ import print_function

from django.core.management.base import BaseCommand

from ... import lib


class Command(BaseCommand):
    """Command definition."""

    help = "Import DMARC aggregated reports."

    def add_arguments(self, parser):
        """Add extra arguments to parser."""
        parser.add_argument(
            "--pipe", action="store_true", default=False,
            help="Import a report from an email given on stdin")
        parser.add_argument(
            "--imap", action="store_true", default=False,
            help="Import a report from an IMAP mailbox")
        parser.add_argument(
            "--host", default="localhost",
            help="IMAP host")
        parser.add_argument(
            "--ssl", action="store_true", default=False,
            help="Connect using SSL"),
        parser.add_argument(
            "--mailbox", default="INBOX",
            help="IMAP mailbox to import reports from")

    def handle(self, *args, **options):
        """Entry point."""
        if options.get("pipe"):
            lib.import_report_from_stdin()
        elif options.get("imap"):
            lib.import_from_imap(options)
        else:
            print("Nothing to do.")



"""Internal library."""

import datetime
import email
import fileinput
import getpass
import imaplib
import zipfile
import gzip
import sys

from defusedxml.ElementTree import fromstring
import pytz.exceptions
import six

from django.db import transaction
from django.utils.encoding import smart_text
from django.utils import timezone

from modoboa.admin import models as admin_models

from . import constants
from . import models

ZIP_CONTENT_TYPES = [
    "application/x-zip-compressed",
    "application/x-zip",
    "application/zip",
    "application/gzip",
    "text/xml",
]


def import_record(xml_node, report):
    """Import a record."""
    record = models.Record(report=report)
    row = xml_node.find("row")
    record.source_ip = row.find("source_ip").text
    record.count = int(row.find("count").text)

    policy_evaluated = row.find("policy_evaluated")
    record.disposition = policy_evaluated.find("disposition").text
    record.dkim_result = policy_evaluated.find("dkim").text
    record.spf_result = policy_evaluated.find("spf").text
    reason = policy_evaluated.find("reason")
    if reason:
        record.reason_type = smart_text(reason.find("type").text)[:14]
        if record.reason_type not in constants.ALLOWED_REASON_TYPES:
            record.reason_type = "other"
        comment = reason.find("comment").text or ""
        record.reason_comment = comment

    identifiers = xml_node.find("identifiers")
    header_from = identifiers.find("header_from").text.split(".")
    domain = None
    while len(header_from) >= 2:
        domain = admin_models.Domain.objects.filter(
            name=".".join(header_from)).first()
        if domain is not None:
            record.header_from = domain
            break
        header_from = header_from[1:]
    if domain is None:
        print("Invalid record found (domain not local)")
        return None

    record.save()
    auth_results = xml_node.find("auth_results")
    for rtype in ["spf", "dkim"]:
        rnode = auth_results.find(rtype)
        if not rnode:
            continue
        models.Result.objects.create(
            record=record, type=rtype, domain=rnode.find("domain").text,
            result=rnode.find("result").text)


@transaction.atomic
def import_report(content):
    """Import an aggregated report."""
    root = fromstring(content, forbid_dtd=True)
    metadata = root.find("report_metadata")
    print(
        "Importing report {} received from {}".format(
            metadata.find("report_id").text,
            metadata.find("org_name").text)
    )
    reporter, created = models.Reporter.objects.get_or_create(
        email=metadata.find("email").text,
        defaults={"org_name": metadata.find("org_name").text}
    )
    qs = models.Report.objects.filter(
        reporter=reporter, report_id=metadata.find("report_id").text)
    if qs.exists():
        print("Report already imported.")
        return
    report = models.Report(reporter=reporter)

    report.report_id = metadata.find("report_id").text
    date_range = metadata.find("date_range")
    report.start_date = timezone.make_aware(
        datetime.datetime.fromtimestamp(int(date_range.find("begin").text))
    )
    report.end_date = timezone.make_aware(
        datetime.datetime.fromtimestamp(int(date_range.find("end").text))
    )

    policy_published = root.find("policy_published")
    for attr in ["domain", "adkim", "aspf", "p", "sp", "pct"]:
        try:
            setattr(
                report, "policy_{}".format(attr),
                policy_published.find(attr).text
            )
        except AttributeError:
            pass
    try:
        report.save()
    except (pytz.exceptions.AmbiguousTimeError):
        print("Report skipped because of invalid date.")
        return
    for record in root.findall("record"):
        import_record(record, report)


def import_archive(archive, content_type=None):
    """Import reports contained inside (file pointer)
    - a zip archive,
    - a gzip file,
    - a xml file.
    """
    if content_type == "text/xml":
        import_report(archive.read())
    elif content_type == "application/gzip":
        with gzip.GzipFile(mode="r", fileobj=archive) as zfile:
            import_report(zfile.read())
    else:
        with zipfile.ZipFile(archive, "r") as zfile:
            for fname in zfile.namelist():
                import_report(zfile.read(fname))


def import_report_from_email(content):
    """Import a report from an email."""
    if isinstance(content, six.string_types):
        msg = email.message_from_string(content)
    else:
        msg = email.message_from_file(content)
    err = False
    for part in msg.walk():
        if part.get_content_type() not in ZIP_CONTENT_TYPES:
            continue
        try:
            fpo = six.BytesIO(part.get_payload(decode=True))
            import_archive(fpo, content_type=part.get_content_type())
        except (OSError, IOError):
            print('Error: the attachment does not match the mimetype')
            err = True
        else:
            fpo.close()
    if err:
        # Return EX_DATAERR code <data format error> available
        # at sysexits.h file
        # (see http://www.postfix.org/pipe.8.html)
        sys.exit(65)


def import_report_from_stdin():
    """Parse a report from stdin."""
    content = six.StringIO()
    for line in fileinput.input([]):
        content.write(line)
    content.seek(0)

    if not content:
        return
    import_report_from_email(content)


def import_from_imap(options):
    """Import reports from an IMAP mailbox."""
    obj = imaplib.IMAP4_SSL if options["ssl"] else imaplib.IMAP4
    conn = obj(options["host"])
    username = raw_input("Username: ")
    password = getpass.getpass(prompt="Password: ")
    conn.login(username, password)
    conn.select(options["mailbox"])
    type, msg_ids = conn.search(None, "ALL")
    for msg_id in msg_ids[0].split():
        typ, content = conn.fetch(msg_id, "(RFC822)")
        for response_part in content:
            if isinstance(response_part, tuple):
                import_report_from_email(response_part[1])
    conn.close()



"""DMARC views."""

import concurrent.futures
import datetime
import tldextract

from dns import resolver, reversename

from django.db.models import Q
from django.utils import timezone
from django.views import generic

from django.contrib.auth import mixins as auth_mixins
from django.utils.translation import ugettext_lazy as _

from modoboa.admin import models as admin_models
from modoboa.parameters import tools as param_tools

from . import models


def insert_record(target, record, name):
    """Add a record."""

    if name not in target:
        target[name] = {}

    if record.source_ip not in target[name]:
        target[name][record.source_ip] = {
            "total": 0,
            "spf": {"pass": 0, "fail": 0},
            "dkim": {"pass": 0, "fail": 0}
        }
    target[name][record.source_ip]["total"] += record.count
    for typ in ["spf", "dkim"]:
        result = getattr(record, "{}_result".format(typ))
        target[name][record.source_ip][typ][result] += record.count


def week_range(year, weeknumber):
    """Return start and end dates of a given week."""
    tz = timezone.get_current_timezone()
    fmt = "%Y-%W-%w"
    start_week = datetime.datetime.strptime(
        "{}-{}-{}".format(year, weeknumber, 1), fmt)
    end_week = datetime.datetime.strptime(
        "{}-{}-{}".format(year, weeknumber, 0), fmt)
    return tz.localize(start_week), tz.localize(end_week)


class DomainReportView(
        auth_mixins.PermissionRequiredMixin,
        generic.TemplateView):
    """ListView for Report."""

    permission_required = "modoboa_dmarc.view_report"
    template_name = "modoboa_dmarc/domain_report.html"

    def get_queryset(self):
        """Filter reports."""
        self.period = self.request.GET.get("period", "")
        if not self.period:
            year, week, day = timezone.now().isocalendar()
            week -= 1
            self.period = "{}-{}".format(year, week)
        else:
            year, week = self.period.split("-")

        self.daterange = week_range(year, week)
        self.domain = admin_models.Domain.objects.get(pk=self.kwargs["pk"])
        qargs = (
            (Q(report__start_date__gte=self.daterange[0],
               report__start_date__lte=self.daterange[1]) |
             Q(report__end_date__gte=self.daterange[0],
               report__end_date__lte=self.daterange[1])) &
            Q(header_from=self.domain)
        )
        return models.Record.objects.select_related().filter(qargs)

    def get_context_data(self, *args, **kwargs):
        """Extra context data."""
        context = super(DomainReportView, self).get_context_data(
            *args, **kwargs)
        qset = self.get_queryset()
        stats = {
            "total": 0,
            "aligned": 0,
            "trusted": 0,
            "failed": 0
        }
        trusted = {}
        threats = {}
        all_records = qset.all()
        dns_names = {}
        if param_tools.get_global_parameter("enable_rlookups"):
            dns_resolver = resolver.Resolver()
            dns_resolver.timeout = 1.0
            dns_resolver.lifetime = 1.0

            def get_domain_name_from_ip(ip):
                addr = reversename.from_address(ip)
                try:
                    resp = dns_resolver.query(addr, "PTR")
                    ext = tldextract.extract(str(resp[0].target))
                    if not ext.suffix:  # invalid PTR record
                        raise resolver.NXDOMAIN()
                    return (ip, '.'.join((ext.domain, ext.suffix)).lower())
                except (resolver.NXDOMAIN, resolver.YXDOMAIN,
                        resolver.NoAnswer, resolver.NoNameservers,
                        resolver.Timeout):
                    return (None, None)

            ips = (r.source_ip for r in all_records)
            with concurrent.futures.ThreadPoolExecutor(max_workers=16) as pool:
                dns_names = {i: n for (i, n) in
                             list(pool.map(get_domain_name_from_ip, ips))}

        for record in all_records:
            stats["total"] += record.count
            name = dns_names.get(record.source_ip, _("Not resolved"))
            if record.dkim_result == "pass" and record.spf_result == "pass":
                stats["aligned"] += record.count
                stats["trusted"] += record.count
                insert_record(trusted, record, name)
            elif record.dkim_result == "pass" or record.spf_result == "pass":
                stats["trusted"] += record.count
                insert_record(trusted, record, name)
            else:
                insert_record(threats, record, name)
                stats["failed"] += record.count

        stats["paligned"] = (
            stats["total"] - (stats["aligned"] + stats["failed"]))
        pie_data = {}
        if stats["total"]:
            pie_data.update({
                "faligned": stats["aligned"] / float(stats["total"]) * 100,
                "paligned": (
                    (stats["trusted"] - stats["aligned"]) /
                    float(stats["total"]) * 100
                ),
                "failed": stats["failed"] / float(stats["total"]) * 100
            })

        context.update({
            "stats": stats, "trusted": trusted, "threats": threats,
            "period": self.period, "daterange": self.daterange,
            "domain": self.domain, "pie_data": pie_data
        })
        return context



# Generated by Django 2.2.9 on 2020-02-27 08:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('modoboa_dmarc', '0002_auto_20180124_2311'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='report',
            options={},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0002_migrate_from_modoboa_admin'),
    ]

    operations = [
        migrations.CreateModel(
            name='Record',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('source_ip', models.GenericIPAddressField()),
                ('count', models.IntegerField()),
                ('disposition', models.CharField(max_length=10, choices=[(b'none', 'None'), (b'quarantine', 'Quarantine'), (b'reject', 'Reject')])),
                ('dkim_result', models.CharField(max_length=9, choices=[(b'none', 'None'), (b'neutral', 'Neutral'), (b'pass', 'Pass'), (b'fail', 'Fail'), (b'temperror', 'Temporary error'), (b'permerror', 'Permanent error'), (b'policy', 'Policy')])),
                ('spf_result', models.CharField(max_length=9, choices=[(b'none', 'None'), (b'neutral', 'Neutral'), (b'pass', 'Pass'), (b'fail', 'Fail'), (b'temperror', 'Temporary error'), (b'permerror', 'Permanent error'), (b'softfail', 'Soft failure')])),
                ('reason_type', models.CharField(max_length=15, blank=True)),
                ('reason_comment', models.CharField(max_length=100, blank=True)),
                ('header_from', models.ForeignKey(to='admin.Domain', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Report',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('report_id', models.CharField(max_length=100)),
                ('start_date', models.DateTimeField()),
                ('end_date', models.DateTimeField()),
                ('policy_domain', models.CharField(max_length=100)),
                ('policy_adkim', models.CharField(max_length=1)),
                ('policy_aspf', models.CharField(max_length=1)),
                ('policy_p', models.CharField(max_length=10)),
                ('policy_sp', models.CharField(max_length=10)),
                ('policy_pct', models.SmallIntegerField()),
            ],
            options={
                'permissions': (('view_report', 'Can view report'),),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Reporter',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('org_name', models.CharField(max_length=100)),
                ('email', models.EmailField(unique=True, max_length=254)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Result',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=4, choices=[(b'dkim', b'DKIM'), (b'spf', b'SPF')])),
                ('domain', models.CharField(max_length=100)),
                ('result', models.CharField(max_length=9)),
                ('record', models.ForeignKey(to='modoboa_dmarc.Record', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='report',
            name='reporter',
            field=models.ForeignKey(to='modoboa_dmarc.Reporter', on_delete=models.CASCADE),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='report',
            unique_together=set([('reporter', 'report_id')]),
        ),
        migrations.AddField(
            model_name='record',
            name='report',
            field=models.ForeignKey(to='modoboa_dmarc.Report', on_delete=models.CASCADE),
            preserve_default=True,
        ),
    ]



"""DMARC tools for Modoboa."""

from django.utils.translation import ugettext_lazy

from modoboa.core.extensions import ModoExtension, exts_pool
from modoboa.parameters import tools as param_tools

from . import __version__
from . import forms


class DmarcExtension(ModoExtension):
    """Extension registration."""

    name = "modoboa_dmarc"
    label = ugettext_lazy("DMARC tools")
    version = __version__
    description = ugettext_lazy(
        "A set of tools to ease DMARC integration"
    )
    url = "dmarc"

    def load(self):
        """Extension loading."""
        param_tools.registry.add("global", forms.ParametersForm, "DMARC")


exts_pool.register_extension(DmarcExtension)



"""Custom template tags."""

import datetime

from collections import OrderedDict

from django import template
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

register = template.Library()


@register.simple_tag
def next_period(period):
    """Return next period."""
    current = period.split("-")
    if int(current[1]) == 52:
        week0 = datetime.datetime.strptime("{}-0-1".format(int(current[0]) + 1 ), "%Y-%W-%w")
        week1 = datetime.datetime.strptime("{}-1-1".format(int(current[0]) + 1 ), "%Y-%W-%w")
        week52 = datetime.datetime.strptime("{}-52-1".format(current[0]), "%Y-%W-%w")
        if week0 == week52 or week0 == week1 :
            parts = (int(current[0]) + 1, 1 )
        else:
            parts = (int(current[0]) + 1, 0 )
    else:
        parts = (current[0], int(current[1]) + 1 )
    return mark_safe("{}-{}".format(parts[0], parts[1]))

@register.simple_tag
def previous_period(period):
    """Return previous period."""
    current = period.split("-")
    if int(current[1]) == 1:
        week0 = datetime.datetime.strptime("{}-0-1".format(current[0]), "%Y-%W-%w")
        week1 = datetime.datetime.strptime("{}-1-1".format(current[0]), "%Y-%W-%w")
        if week0 == week1:
            parts = (int(current[0]) - 1, 52 )
        else:
            parts = (current[0], 0 )
    elif int(current[1]) == 0:
        week0 = datetime.datetime.strptime("{}-0-1".format(current[0]), "%Y-%W-%w")
        week52 = datetime.datetime.strptime("{}-52-1".format(int(current[0]) - 1 ), "%Y-%W-%w")
        if week0 == week52:
            parts = (int(current[0]) - 1, 51 )
        else:
            parts = (int(current[0]) - 1, 52 )
    else:
        parts = (current[0], int(current[1]) - 1 )
    return mark_safe("{}-{}".format(parts[0], parts[1]))


@register.filter
def domain_sorted_items(domain_dict):
    """Return a list of tuples ordered alphabetically by domain names."""
    if isinstance(domain_dict, dict):
        sorted_domain_dict = OrderedDict(
            sorted(domain_dict.items(), key=lambda t: t[0]))
        unresolved_label = _("Not resolved")
        unresolved = sorted_domain_dict.pop(unresolved_label, None)
        if unresolved:
            sorted_domain_dict[unresolved_label] = unresolved
        return sorted_domain_dict.items()

    raise ValueError("domain_dict is not a dict")



"""Management command tests."""

from modoboa.admin import factories as admin_factories
from modoboa.lib.tests import ModoTestCase

from . import mixins
from .. import models


class ManagementCommandTestCase(mixins.CallCommandMixin, ModoTestCase):
    """Test management command."""

    @classmethod
    def setUpTestData(cls):
        super(ManagementCommandTestCase, cls).setUpTestData()
        cls.domain = admin_factories.DomainFactory(name="ngyn.org")

    def test_import_from_archive(self):
        """Import report from archive."""
        self.import_reports()
        self.import_fail_reports()
        self.assertTrue(self.domain.record_set.exists())
        self.assertTrue(
            models.Reporter.objects.filter(
                org_name="FastMail Pty Ltd").exists())



"""Custom forms."""

from django import forms
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from modoboa.lib import form_utils
from modoboa.parameters import forms as param_forms


class ReportOptionsForm(forms.Form):
    """Report display options."""

    current_year = forms.IntegerField(widget=forms.widgets.HiddenInput)
    current_week = forms.IntegerField()
    query = forms.ChoiceField(
        choices=[("previous", "Previous"), ("next", "Next")])
    resolve_hostnames = forms.BooleanField(
        initial=False, required=False)

    def __init__(self, *args, **kwargs):
        """Constructor."""
        super(ReportOptionsForm, self).__init__(*args, **kwargs)
        if not args:
            year, week, day = timezone.now().isocalendar()
            self.fields["current_year"].initial = year
            self.fields["current_week"].initial = week


class ParametersForm(param_forms.AdminParametersForm):
    """Extension settings."""

    app = "modoboa_dmarc"

    qsettings_sep = form_utils.SeparatorField(label=_("DNS settings"))

    enable_rlookups = form_utils.YesNoField(
        label=_("Enable reverse lookups"),
        initial=False,
        help_text=_(
            "Enable reverse DNS lookups (reports will be longer to display)"
        )
    )



"""DMARC urls."""

from django.urls import path

from . import views

app_name = "modoboa_dmarc"

urlpatterns = [
    path("domains/<int:pk>/", views.DomainReportView.as_view(),
         name="domain_report"),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-24 23:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('modoboa_dmarc', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='record',
            name='disposition',
            field=models.CharField(choices=[('none', 'None'), ('quarantine', 'Quarantine'), ('reject', 'Reject')], max_length=10),
        ),
        migrations.AlterField(
            model_name='record',
            name='dkim_result',
            field=models.CharField(choices=[('none', 'None'), ('neutral', 'Neutral'), ('pass', 'Pass'), ('fail', 'Fail'), ('temperror', 'Temporary error'), ('permerror', 'Permanent error'), ('policy', 'Policy')], max_length=9),
        ),
        migrations.AlterField(
            model_name='record',
            name='spf_result',
            field=models.CharField(choices=[('none', 'None'), ('neutral', 'Neutral'), ('pass', 'Pass'), ('fail', 'Fail'), ('temperror', 'Temporary error'), ('permerror', 'Permanent error'), ('softfail', 'Soft failure')], max_length=9),
        ),
        migrations.AlterField(
            model_name='result',
            name='type',
            field=models.CharField(choices=[('dkim', 'DKIM'), ('spf', 'SPF')], max_length=4),
        ),
    ]



"""Views tests."""

from django.urls import reverse

from modoboa.admin import factories as admin_factories
from modoboa.core import models as core_models
from modoboa.lib.tests import ModoTestCase

from . import mixins


class DMARCViewsTestCase(mixins.CallCommandMixin, ModoTestCase):
    """Views test cases."""

    @classmethod
    def setUpTestData(cls):
        """Create test data."""
        super(DMARCViewsTestCase, cls).setUpTestData()
        cls.domain = admin_factories.DomainFactory(name="ngyn.org")

    def test_domainlist_view(self):
        """Test domain list view."""
        self.import_reports()
        user = core_models.User.objects.get(username="admin")
        self.client.force_login(user)
        url = reverse("admin:_domain_list")
        response = self.ajax_get(url)
        url = reverse("modoboa_dmarc:domain_report", args=[self.domain.pk])
        self.assertIn(url, response["rows"])

    def test_domainreport_view(self):
        """Test domain report view."""
        self.import_reports()
        user = core_models.User.objects.get(username="admin")
        self.client.force_login(user)
        url = reverse("modoboa_dmarc:domain_report", args=[self.domain.pk])
        response = self.client.get("{}?period=2015-26".format(url))
        self.assertContains(response, "'Failed', 100.0")

    def test_domainreport_view_week0(self):
        """Test domain report view for week 0."""
        self.import_reports()
        user = core_models.User.objects.get(username="admin")
        self.client.force_login(user)
        url = reverse("modoboa_dmarc:domain_report", args=[self.domain.pk])
        response = self.client.get("{}?period=2019-0".format(url))
        self.assertContains(response, "Dec. 31, 2018")

    def test_domainreport_view_week1(self):
        """Test domain report view for week 1."""
        self.import_reports()
        user = core_models.User.objects.get(username="admin")
        self.client.force_login(user)
        url = reverse("modoboa_dmarc:domain_report", args=[self.domain.pk])
        response = self.client.get("{}?period=2019-1".format(url))
        self.assertContains(response, "Jan. 7, 2019")

    def test_domainreport_view_week52(self):
        """Test domain report view for week 52."""
        self.import_reports()
        user = core_models.User.objects.get(username="admin")
        self.client.force_login(user)
        url = reverse("modoboa_dmarc:domain_report", args=[self.domain.pk])
        response = self.client.get("{}?period=2018-52".format(url))
        self.assertContains(response, "Dec. 30, 2018")




"""Django signal handlers for modoboa_dmarc."""

from django.urls import reverse
from django.utils.translation import ugettext as _
from django.dispatch import receiver

from modoboa.admin import signals as admin_signals

from . import models


@receiver(admin_signals.extra_domain_actions)
def dmarc_domain_actions(sender, user, domain, **kwargs):
    """Return a link to access domain report."""
    if not models.Record.objects.filter(header_from=domain).exists():
        return []
    return [{
        "name": "dmarc_report",
        "url": reverse("modoboa_dmarc:domain_report", args=[domain.pk]),
        "title": _("Show DMARC report for {}").format(domain.name),
        "img": "fa fa-pie-chart"
    }]
